package industries.sunshine.planningpoker

import cats.effect._
import cats.data.Kleisli
import cats.data.OptionT
import org.http4s.Request
import industries.sunshine.planningpoker.common.Models.PlayerID
import java.util.UUID
import industries.sunshine.planningpoker.common.Models.RoomID

trait Auth {

  /** for middleware that converts sessionId into PlayerId
    */
  def authUser: Kleisli[[A] =>> cats.data.OptionT[cats.effect.IO, A], Request[
    cats.effect.IO
  ], (PlayerID, RoomID)]

  /** Get sessionId for accessing a room
    * @param roomPassword
    *   \- requirement to get access to the room
    */
  def accessRoom(
      roomName: String,
      roomPassword: String,
      nickName: String
  ): IO[Either[Unit, Long]]

  def leaveRoom(
    sessionId: Long
  ): IO[Unit]

}

object Auth {
  def pureBadStub = new Auth {
    override def authUser =
      Kleisli((req: Request[IO]) =>
        OptionT.liftF(IO(println(s"authUser: $req")) >> IO(PlayerID(14) -> RoomID(101)))
      )
    override def accessRoom(
        roomName: String,
        roomPassword: String,
        nickName: String
    ) =
      IO(println(s"> access room for $roomName $roomPassword $nickName, to return stub 111")) >> IO.pure(Right(111L))

    override def leaveRoom(sessionId: Long): IO[Unit] =
      IO(s"got request to leave for $sessionId")
  }

  type SessionsMap = Map[Long, (RoomID, PlayerID)]
  val sessionsRef = Ref.of[IO, SessionsMap](Map.empty)
  val roomPasswordsRef = Ref.of[IO, Map[Long, String]](Map.empty)

  def apply(): IO[Auth] =
    for {
      store <- sessionsRef
      roomPasswords <- roomPasswordsRef
    } yield new Auth {
      override def authUser = Kleisli { (req: Request[IO]) =>
        {
          ???
        }
      }
      override def accessRoom(
          roomName: String,
          roomPassword: String,
          nickName: String
      ): IO[Either[Unit, Long]] = ???

      override def leaveRoom(sessionId: Long): IO[Unit] = ???
    }
}
